# -*- coding: utf-8 -*-
from __future__ import unicode_literals


import warnings

from glitter.blockadmin.blocks import BlockAdmin, site, StackedInline, TabularInline

from .models import BaseBlock  # noqa


BlockModelAdmin = BlockAdmin

__all__ = ['site', 'BlockModelAdmin', 'StackedInline', 'TabularInline']


warnings.warn(
    "BlockModelAdmin has been moved to blockadmin.blocks.BlockAdmin",
    DeprecationWarning,
    stacklevel=2
)



"""
These template tags allows the reusability of django-admin resources between
version 1.8 and 1.8+ of django.
"""
from django import template, VERSION
from django.contrib.staticfiles.templatetags.staticfiles import static
register = template.Library()


@register.simple_tag
def jquery_min():
    "Return the path to jquery.min.js"
    if VERSION >= (1, 9):
        url = static('admin/js/vendor/jquery/jquery.min.js')
    else:
        url = static('admin/js/jquery.min.js')

    return url



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from django.conf.urls import url
from django.forms.forms import pretty_name
from django.http import HttpResponse, HttpResponseForbidden, JsonResponse
from django.template import Context
from django.template.loader import get_template

from glitter.assets.forms import ImageForm
from glitter.assets.models import Image
from glitter.assets.widgets import LIMIT_IMAGES_TO, ImageRelatedFieldWidgetWrapper, ImageSelect
from glitter.blockadmin import blocks

from .forms import ImageBlockForm
from .models import ImageBlock


class ImageBlockAdmin(blocks.BlockAdmin):
    form = ImageBlockForm

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Hook for specifying the form Field instance for a given database Field
        instance.

        If kwargs are given, they're passed to the form Field's constructor.
        """
        formfield = super(ImageBlockAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if db_field.name == 'image':
            formfield.widget = ImageRelatedFieldWidgetWrapper(
                ImageSelect(), db_field.rel, self.admin_site, can_add_related=True,
                can_change_related=True,
            )
        return formfield

    def get_urls(self):
        urls = super(ImageBlockAdmin, self).get_urls()
        app_label, model_name = self.model._meta.app_label, self.model._meta.model_name
        image_block_urls = [
            url(r'^get-lazy-images/$', self.get_lazy_images, name='get-lazy-images'),
            url(
                r'^drop-image/$',
                self.drop_image,
                name='{app_label}_{model_name}_drop_image'.format(
                    app_label=app_label, model_name=model_name
                )
            ),
        ]
        return image_block_urls + urls

    def get_lazy_images(self, request):
        last_image_id = request.GET.get('last_image_id', None)
        if last_image_id and last_image_id.isdigit():
            images = Image.objects.filter(
                id__lt=last_image_id
            ).order_by(
                '-created_at', 'modified_at', 'title'
            )[:LIMIT_IMAGES_TO]
            template = get_template('glitter/blocks/includes/lazy_images.html')
            context = Context({'images': images})
            html = template.render(context)
            return JsonResponse({'html': html, 'last_image_id': last_image_id})
        else:
            response = JsonResponse({'error': 'No last image id passed'})
            response.status_code = 400

        return response

    def drop_image(self, request):

        permission_name = '{}.edit_{}'.format(self.opts.app_label, self.opts.model_name)
        if not request.user.has_perm(permission_name):
            return HttpResponseForbidden('403 Forbidden', content_type='text/html')

        form = ImageForm(request.POST, request.FILES or None)
        if form.is_valid():
            filename, ext = os.path.splitext(form.files['file'].name)
            filename = pretty_name(filename)

            image = form.save(commit=False)
            image.title = filename
            image.save()
            return JsonResponse({'image_id': image.id, 'filename': filename})
        else:
            message = ''
            for field_name, error_list in form.errors.items():
                message = '{field} - {message}'.format(
                    field=field_name.capitalize(), message=''.join(error_list)
                )
                return HttpResponse(message, status=403)


blocks.site.register(ImageBlock, ImageBlockAdmin)
blocks.site.register_block(ImageBlock, 'Common')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from glitter.mixins import GlitterMixin
from glitter.models import Version
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey
from taggit.managers import TaggableManager

from .validators import validate_page_url


class PageManager(TreeManager):
    def published(self):
        return self.filter(published=True).exclude(current_version=None)

    def unpublished(self):
        return self.filter(published=True, current_version__isnull=True)


@python_2_unicode_compatible
class Page(MPTTModel, GlitterMixin):
    url = models.CharField('URL', max_length=100, unique=True, validators=[validate_page_url])
    title = models.CharField(max_length=100)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
    login_required = models.BooleanField(default=False)
    show_in_navigation = models.BooleanField(default=True, db_index=True)
    unpublished_count = models.PositiveIntegerField(default=0, editable=False)
    tags = TaggableManager(blank=True)

    glitter_app_name = models.CharField(
        verbose_name='Glitter App', max_length=255, db_index=True, blank=True
    )

    objects = PageManager()

    def __str__(self):
        return '%s -- %s' % (self.title, self.url)

    class Meta(GlitterMixin.Meta):
        verbose_name = 'page'
        ordering = ('url',)
        permissions = (
            ('view_protected_page', 'Can view protected page'),
        )

    def get_absolute_url(self):
        return self.url

    def save(self, *args, **kwargs):
        # Find the number of unpublished pages
        content_type = ContentType.objects.get_for_model(self)
        unpublished_pages = Version.objects.filter(
            content_type=content_type, object_id=self.id
        ).exclude(version_number__isnull=True)

        if self.current_version:
            unpublished_pages = unpublished_pages.filter(
                version_number__gt=self.current_version.version_number
            )

        self.unpublished_count = unpublished_pages.count()

        super(Page, self).save(*args, **kwargs)

    @property
    def is_visible(self):
        """
        Return a boolean if the page is visible in navigation.

        Pages must have show in navigation set. Regular pages must be published (published and
        have a current version - checked with `is_published`), pages with a glitter app associated
        don't need any page versions.
        """
        if self.glitter_app_name:
            visible = self.show_in_navigation
        else:
            visible = self.show_in_navigation and self.is_published

        return visible



from django.conf import settings
from importlib import import_module


class GlitterApp(object):
    """
    A simple class to store Glitter App config options.
    """
    def __init__(self, name, url_conf, namespace=None):
        self.name = name
        self.url_conf = url_conf
        self.namespace = namespace

    def __str__(self):
        return self.name


class GlitterAppPool(object):
    """
    An interface to the Glitter App configs in the current project.

    Will lazily discover Glitter App configs when needed.
    """
    def __init__(self):
        self.glitter_apps = {}
        self.discovered = False

    def get_glitter_app(self, glitter_app_name):
        """
        Retrieve the Glitter App config for a specific Glitter App.
        """
        if not self.discovered:
            self.discover_glitter_apps()

        try:
            glitter_app = self.glitter_apps[glitter_app_name]
            return glitter_app
        except KeyError:
            return None

    def discover_glitter_apps(self):
        """
        Find all the Glitter App configurations in the current project.
        """
        for app_name in settings.INSTALLED_APPS:
            module_name = '{app_name}.glitter_apps'.format(app_name=app_name)
            try:
                glitter_apps_module = import_module(module_name)
                if hasattr(glitter_apps_module, 'apps'):
                    self.glitter_apps.update(glitter_apps_module.apps)
            except ImportError:
                pass

        self.discovered = True

    def get_glitter_apps(self):
        if not self.discovered:
            self.discover_glitter_apps()
        return self.glitter_apps


glitter_app_pool = GlitterAppPool()



from __future__ import unicode_literals

from django.apps import AppConfig


class RemindersConfig(AppConfig):
    name = 'glitter.reminders'
    label = 'glitter_reminders'
    verbose_name = 'reminders'



from __future__ import unicode_literals

from django.conf import settings
from django.utils import timezone

from .models import PublishAction


def process_actions(action_ids=None):
    """
    Process actions in the publishing schedule.

    Returns the number of actions processed.
    """
    actions_taken = 0
    action_list = PublishAction.objects.prefetch_related(
        'content_object',
    ).filter(
        scheduled_time__lte=timezone.now(),
    )

    if action_ids is not None:
        action_list = action_list.filter(id__in=action_ids)

    for action in action_list:
        action.process_action()
        action.delete()
        actions_taken += 1

    return actions_taken


def celery_enabled():
    """
    Return a boolean if Celery tasks are enabled for this app.

    If the ``GLITTER_PUBLISHER_CELERY`` setting is ``True`` or ``False`` - then that value will be
    used. However if the setting isn't defined, then this will be enabled automatically if Celery
    is installed.
    """
    enabled = getattr(settings, 'GLITTER_PUBLISHER_CELERY', None)

    if enabled is None:
        try:
            import celery  # noqa
            enabled = True
        except ImportError:
            enabled = False

    return enabled



from __future__ import unicode_literals

from django.test import TestCase

from .factories import PageFactory, PageVersionFactory


class TestGlitterMixinPublished(TestCase):
    def test_published_true(self):
        page = PageFactory()

        self.assertFalse(page.is_published)

    def test_all_false(self):
        page = PageFactory(published=False)

        self.assertFalse(page.is_published)

    def test_unpublished(self):
        page_version = PageVersionFactory()
        page = page_version.content_object

        self.assertFalse(page.is_published)

    def test_all_true(self):
        page_version = PageVersionFactory(version_number=1, set_version=True)
        page = page_version.content_object

        self.assertTrue(page.is_published)



# -*- coding: utf-8 -*-

import django
from django.contrib.admin.sites import AdminSite
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.core.urlresolvers import reverse
from django.forms.widgets import Select


class AddBlockSelect(Select):
    def render_options(self, choices, selected_choices):
        header = '<option value="" disabled="" selected="">Add a block:</option>'
        options = super(AddBlockSelect, self).render_options(choices, selected_choices)
        return header + options


class ChooseColumnSelect(Select):
    def render_options(self, choices, selected_choices):
        header = '<option value="" disabled="">Choose column:</option>'
        options = super(ChooseColumnSelect, self).render_options(choices, selected_choices)
        return header + options


class MoveBlockSelect(Select):
    def render_options(self, choices, selected_choices):
        header = '<option value="" disabled="" selected="">Move block:</option>'
        options = super(MoveBlockSelect, self).render_options(choices, selected_choices)
        return header + options


class CustomRelatedFieldWidgetWrapper(RelatedFieldWidgetWrapper):
    def get_related_url(self, info, action, *args):
        current_app = AdminSite()
        if django.VERSION >= (1, 9):
            current_app = None
        viewname = "admin:%s_%s_%s" % (info + (action,))
        return reverse(viewname, current_app=current_app, args=args)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .forms import LatestTweetsBlockForm
from .models import LatestTweetsBlock


class LatestTweetsBlockAdmin(blocks.BlockAdmin):
    form = LatestTweetsBlockForm


blocks.site.register(LatestTweetsBlock, LatestTweetsBlockAdmin)
blocks.site.register_block(LatestTweetsBlock, 'App Blocks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .models import SampleModelWithInlinesBlock, SampleInline


class SampleInlineAdmin(blocks.StackedInline):
    model = SampleInline


class SampleModelWithInlinesBlockAdmin(blocks.BlockAdmin):
    inlines = [SampleInlineAdmin]


blocks.site.register(SampleModelWithInlinesBlock, SampleModelWithInlinesBlockAdmin)
blocks.site.register_block(SampleModelWithInlinesBlock, 'Sample Blocks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .models import CallToActionBlock


blocks.site.register(CallToActionBlock)
blocks.site.register_block(CallToActionBlock, 'Common')



from __future__ import unicode_literals

from django.core.management.base import BaseCommand

from glitter.publisher.utils import process_actions


class Command(BaseCommand):
    help = 'Process scheduled publishing for Glitter objects'

    requires_system_checks = False

    def handle(self, **options):
        actions_taken = process_actions()
        self.stdout.write('Actions taken: {}'.format(actions_taken))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_carousel', '0005_carousel_required'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='carouselimage',
            options={'ordering': ('position', 'id')},
        ),
        migrations.AlterModelOptions(
            name='imageonlycarouselimage',
            options={'ordering': ('position', 'id')},
        ),
        migrations.AddField(
            model_name='carouselimage',
            name='position',
            field=models.PositiveIntegerField(db_index=True, default=0),
        ),
        migrations.AddField(
            model_name='imageonlycarouselimage',
            name='position',
            field=models.PositiveIntegerField(db_index=True, default=0),
        ),
    ]



from django.core.validators import ValidationError
from django.utils import timezone


def future_date(date):
    """
    Validator which only allows current/future datetimes.
    """
    if date < timezone.now():
        raise ValidationError("Can't set a scheduled time in the past")



from __future__ import unicode_literals

from django.apps import AppConfig


class PublisherConfig(AppConfig):
    name = 'glitter.publisher'
    label = 'glitter_publisher'
    verbose_name = 'Publisher'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def remove_empty_blocks(apps, schema_editor):
    ContentBlock = apps.get_model('glitter', 'ContentBlock')
    ContentBlock.objects.filter(object_id=None).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0002_object_id_not_required'),
    ]

    operations = [
        migrations.RunPython(remove_empty_blocks, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_carousel', '0004_delete_empty_blocks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='carouselblock',
            name='carousel',
            field=models.ForeignKey(to='glitter_carousel.Carousel', on_delete=django.db.models.deletion.PROTECT),
        ),
        migrations.AlterField(
            model_name='imageonlycarouselblock',
            name='carousel',
            field=models.ForeignKey(to='glitter_carousel.ImageOnlyCarousel', on_delete=django.db.models.deletion.PROTECT),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from functools import update_wrapper

from django.conf import settings
from django.contrib import messages
from django.contrib.admin import ModelAdmin
from django.contrib.admin.options import InlineModelAdmin
from django.contrib.admin.sites import AdminSite
from django.contrib.admin.utils import unquote, quote
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import resolve, reverse
from django.db.models.base import ModelBase
from django.http import Http404, HttpResponseBadRequest, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.defaultfilters import slugify
from django.template.response import TemplateResponse
from django.utils import six
from django.utils.encoding import force_text
from django.utils.html import escape
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect

from glitter.models import ContentBlock, Version
from glitter.page import Glitter
from glitter.templates import get_layout
from glitter.utils import JSONEncoderForHTML


class BlockAdminSite(AdminSite):
    change_form_template = 'blockadmin/change_form.html'

    def __init__(self, *args, **kwargs):
        # All blocks can be registered to this admin site
        self.block_list = {}

        super(BlockAdminSite, self).__init__(*args, **kwargs)

    # Use the block admin class by default
    def register(self, model_or_iterable, admin_class=None, **options):
        if not admin_class:
            admin_class = BlockAdmin

        category = options.pop('category', None)
        if category:
            self.register_block(model_or_iterable, category)

        super(BlockAdminSite, self).register(model_or_iterable, admin_class, **options)

    # Blocks from the site or other apps can be registered
    def register_block(self, block_or_iterable, category):
        if category not in self.block_list:
            self.block_list[category] = []

        if issubclass(block_or_iterable.__class__, ModelBase):
            self.block_list[category].append(block_or_iterable)
        else:
            for block in block_or_iterable:
                self.block_list[category].append(block)

    def unregister_block(self, block, category):
        try:
            self.block_list[category].remove(block)
        except ValueError:
            pass

    # Remove all of the default admin URLs, we only want the model views for
    # the block admin.
    def get_urls(self):
        from django.conf.urls import url, include

        if settings.DEBUG:
            self.check_dependencies()

        def wrap(view, cacheable=False):
            def wrapper(*args, **kwargs):
                return self.admin_view(view, cacheable)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        # Admin-site-wide views.
        urlpatterns = [
            url(r'^jsi18n/$', wrap(self.i18n_javascript, cacheable=True), name='jsi18n'),
        ]

        # Add in each model's views.
        for model, model_admin in six.iteritems(self._registry):
            urlpatterns += [
                url(r'^%s/%s/' % (model._meta.app_label, model._meta.model_name),
                    include(model_admin.urls))
            ]
        return urlpatterns

    # We have no logout here, so just raise PermissionDenied if needed
    def admin_view(self, view, cacheable=False):
        def inner(request, *args, **kwargs):
            if not self.has_permission(request):
                raise PermissionDenied
            return view(request, *args, **kwargs)
        if not cacheable:
            inner = never_cache(inner)
        # We add csrf_protect here so this function can be used as a utility
        # function for any view, without having to repeat 'csrf_protect'.
        inner = csrf_protect(inner)
        return update_wrapper(inner, view)

    def get_app_list(self, request):
        return {}


class BlockAdmin(ModelAdmin):
    # Keep block admin change forms in the blockadmin template directory
    @property
    def change_form_template(self):
        opts = self.model._meta
        app_label = opts.app_label

        return (
            "blockadmin/%s/%s/change_form.html" % (app_label, opts.object_name.lower()),
            "blockadmin/%s/change_form.html" % app_label,
            "blockadmin/change_form.html"
        )

    # We only want the change form for the block admin
    def get_urls(self):
        from django.conf.urls import url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = [
            url(r'^add/(?P<version_id>\d+)/$', wrap(self.add_view), name='%s_%s_add' % info),
            url(r'^(.+)/continue/$', wrap(self.continue_view), name='%s_%s_continue' % info),
            url(r'^(.+)/$', wrap(self.change_view), name='%s_%s_change' % info),
        ]
        return urlpatterns

    def log_change(self, request, object, message):
        pass

    def response_rerender(self, request, obj, template, extra_context=None):
        request.current_app = self.admin_site.name
        content_block = obj.content_block
        version = content_block.obj_version

        glitter = Glitter(version, request=request)
        columns = glitter.render(edit_mode=True, rerender=True)

        context = {
            'column': slugify(content_block.column),
            'rendered_json': JSONEncoderForHTML().encode({
                'content': columns[content_block.column],
            }),
        }

        if extra_context is not None:
            context.update(extra_context)

        return TemplateResponse(request, template, context)

    # A redirect back to the edit view
    def continue_view(self, request, object_id):
        obj = self.get_object(request, unquote(object_id))

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {
                'name': force_text(self.opts.verbose_name),
                'key': escape(object_id)},
            )

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        opts = self.opts.app_label, self.opts.model_name
        change_url = reverse(
            'admin:%s_%s_change' % opts,
            args=(object_id,),
            current_app=self.admin_site.name)

        # This template will rerender the
        return self.response_rerender(request, obj, 'blockadmin/continue.html', {
            'change_url': change_url,
        })

    def get_version(self, request):
        # As this is the Django admin and we can't just set self.version_id due to thread safety
        # issues, we need to find the version ID of the page when adding a new block. It'll be
        # accessible in the URL - although this is a very hacky way to access it.
        func, args, kwargs = resolve(request.path)
        version_id = kwargs.get('version_id', None)

        if version_id is None:
            # If a version_id kwarg isn't given - we're probably being called by change_view, so
            # there isn't a Version for this request
            return None
        else:
            return Version.objects.get(id=version_id)

    def has_glitter_edit_permission(self, request, obj):
        """
        Return a boolean if a user has edit access to the glitter object/page this object is on.
        """

        # We're testing for the edit permission here with the glitter object - not the current
        # object, not the change permission. Once a user has edit access to an object they can edit
        # all content on it.
        permission_name = '{}.edit_{}'.format(
            obj._meta.app_label, obj._meta.model_name,
        )
        has_permission = (
            request.user.has_perm(permission_name) or
            request.user.has_perm(permission_name, obj=obj)
        )
        return has_permission

    def has_add_permission(self, request):
        # Find the glitter object to see if they've got permission to edit that
        version = self.get_version(request=request)

        # If a version can't be found for this request, it's probably being called by change_view.
        # In this case we just return True as it shouldn't matter.
        if version is None:
            return True

        glitter_obj = version.content_object

        # Use the glitter object for permission testing
        has_permission = self.has_glitter_edit_permission(request, obj=glitter_obj)
        return has_permission

    def has_change_permission(self, request, obj=None):
        # This shouldn't happen - but given that we need to find out the permissions for the
        # glitter object and not just this content block, just fail early incase something goes
        # wrong.
        if obj is None:
            return False

        # Find the glitter object to see if they've got permission to edit that
        version = obj.content_block.obj_version
        glitter_obj = version.content_object

        # Use the glitter object for permission testing
        has_permission = self.has_glitter_edit_permission(request, obj=glitter_obj)
        return has_permission

    def add_view(self, request, version_id, form_url='', extra_context=None):
        version = get_object_or_404(Version, id=version_id)

        # Version must not be saved, and must belong to this user
        if version.version_number or version.owner != request.user:
            raise PermissionDenied

        # Need to ensure that new blocks go into valid columns
        template_obj = get_layout(template_name=version.template_name)
        column_choices = list(template_obj._meta.columns)

        if request.GET.get('column') not in column_choices:
            return HttpResponseBadRequest('Invalid column')

        return self.changeform_view(request, None, form_url, extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        """The 'change' admin view for this model."""

        obj = self.get_object(request, unquote(object_id))

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {
                'name': force_text(self.opts.verbose_name),
                'key': escape(object_id),
            })

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        content_block = obj.content_block
        version = content_block.obj_version

        # Version must not be saved, and must belong to this user
        if version.version_number or version.owner != request.user:
            raise PermissionDenied

        return super(BlockAdmin, self).change_view(request, object_id, form_url, extra_context)

    def response_change(self, request, obj):
        """Determine the HttpResponse for the change_view stage."""
        opts = self.opts.app_label, self.opts.model_name
        pk_value = obj._get_pk_val()

        if '_continue' in request.POST:
            msg = _(
                'The %(name)s block was changed successfully. You may edit it again below.'
            ) % {'name': force_text(self.opts.verbose_name)}

            self.message_user(request, msg, messages.SUCCESS)

            # We redirect to the save and continue page, which updates the
            # parent window in javascript and redirects back to the edit page
            # in javascript.
            return HttpResponseRedirect(reverse(
                'admin:%s_%s_continue' % opts,
                args=(pk_value,),
                current_app=self.admin_site.name
            ))

        # Update column and close popup - don't bother with a message as they won't see it
        return self.response_rerender(request, obj, 'admin/glitter/update_column.html')

    def save_model(self, request, obj, form, change):
        super(BlockAdmin, self).save_model(request, obj, form, change)

        # Whenever we're adding a new block to a page, we'll need to create a ContentBlock to go
        # along with it
        if change is False:
            version = self.get_version(request=request)

            # Create a content block to link the version and the block
            content_block = ContentBlock(
                obj_version=version,
                column=request.GET['column'],
                content_type=ContentType.objects.get_for_model(obj),
                object_id=obj.id,
            )

            if request.GET.get('top', '').lower() == 'true':
                # User wants block at the top of the column, so set the position or it'll end up
                # being autosaved to the end of the column
                first_block = ContentBlock.objects.filter(
                    obj_version=version, column=content_block.column,
                ).first()

                if first_block is not None:
                    content_block.position = first_block.position - 1

            content_block.save()

            obj.content_block = content_block
            obj.save(update_fields=['content_block'])

    def response_add(self, request, obj, post_url_continue=None):
        opts = obj._meta
        pk_value = obj._get_pk_val()

        # Save and continue editing - continue with the iframe
        if '_continue' in request.POST:
            msg = _('The block was added successfully. You may edit it again below.')
            self.message_user(request, msg, messages.SUCCESS)

            # We redirect to the save and continue page, which updates the
            # parent window in javascript and redirects back to the edit page
            # in javascript.
            post_url_continue = reverse(
                'block_admin:%s_%s_continue' % (opts.app_label, opts.model_name),
                args=(quote(pk_value),),
                current_app=self.admin_site.name
            )
            return HttpResponseRedirect(post_url_continue)

        # Update column and close popup - don't bother with a message as they won't see it
        return self.response_rerender(request, obj, 'admin/glitter/update_column.html')


class InlineBlockAdmin(InlineModelAdmin):
    # For inline objects we always allow adding/editing/deleting. This is an additional check done
    # after the permissions for the object has been checked - everyone needs to be able to edit the
    # same content on the same page, so just allow all inlines.

    def has_add_permission(self, request):
        return True

    def has_change_permission(self, request, obj=None):
        return True

    def has_delete_permission(self, request, obj=None):
        return True


class StackedInline(InlineBlockAdmin):
    template = 'admin/edit_inline/stacked.html'


class TabularInline(InlineBlockAdmin):
    template = 'admin/edit_inline/tabular.html'

site = BlockAdminSite(name='block_admin')


def register(model, admin=None, category=None):
    """ Decorator to registering you Admin class. """
    def _model_admin_wrapper(admin_class):

        site.register(model, admin_class=admin_class)

        if category:
            site.register_block(model, category)

        return admin_class
    return _model_admin_wrapper



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .models import DefinitionList, DefinitionListInline


class DefinitionListInlineAdmin(blocks.TabularInline):
    min_num = 1
    extra = 0
    model = DefinitionListInline


class DefinitionListAdmin(blocks.BlockAdmin):
    inlines = [DefinitionListInlineAdmin]


blocks.site.register(DefinitionList, DefinitionListAdmin)
blocks.site.register_block(DefinitionList, 'Common')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import glitter.publisher.validators
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PublishAction',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('scheduled_time', models.DateTimeField(validators=[glitter.publisher.validators.future_date])),
                ('publish_version', models.IntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=django.db.models.deletion.PROTECT)),
            ],
            options={
                'ordering': ('scheduled_time', 'id'),
                'verbose_name': 'Scheduled publishing',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def remove_empty_blocks(apps, schema_editor):
    HTML = apps.get_model('glitter_html', 'HTML')
    ContentBlock = apps.get_model('glitter', 'ContentBlock')

    empty_blocks = HTML.objects.filter(content='')
    content_block_ids = empty_blocks.values_list('content_block_id', flat=True)

    ContentBlock.objects.filter(id__in=content_block_ids).delete()
    empty_blocks.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_html', '0001_initial'),
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(remove_empty_blocks, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.template.loader import render_to_string

from glitter.templates import get_layout


def banner_view(block, request, rerender, content_block, block_classes):
    css_classes = ' '.join(block_classes)

    # Find the column
    layout = get_layout(template_name=content_block.obj_version.template_name)
    column = layout._meta.columns[content_block.column]

    banner_inlines = None
    if block:
        banner_inlines = block.bannerinline_set.select_related('banner__image').all()

    template_name = 'glitter/blocks/%s.html' % content_block.content_type.model
    context = {
        'content_block': content_block,
        'css_classes': css_classes,
        'object': block,
        'column': column,
        'banner_inlines': banner_inlines}
    rendered = render_to_string(template_name, context, request=request)
    return rendered



from __future__ import unicode_literals

from django import forms
from django.contrib.contenttypes.admin import GenericStackedInline

from .forms import object_version_choices
from .models import PublishAction


class ActionInline(GenericStackedInline):
    model = PublishAction
    fields = ('scheduled_time', 'publish_version')
    extra = 0

    def get_formset(self, request, obj=None, form=None, **kwargs):
        BaseFormset = super(ActionInline, self).get_formset(request, obj, **kwargs)

        class ActionFormset(BaseFormset):
            """
            Customised formset to save the user who has created/updated the action.
            """
            def save_new(self, form, commit):
                obj = super(ActionFormset, self).save_new(form, commit=False)
                obj.user = request.user
                obj.save()
                return obj

            def save_existing(self, form, instance, commit):
                obj = super(ActionFormset, self).save_existing(form, instance, commit=False)
                obj.user = request.user
                obj.save()
                return obj

        # Customised widget which limits the users choices to versions which have been saved for
        # this object.
        ActionFormset.form.base_fields['publish_version'].widget = forms.widgets.Select(
            choices=object_version_choices(obj=obj),
        )

        return ActionFormset



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import glitter.assets.fields


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_text_image', '0002_delete_empty_blocks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='textimageblock',
            name='image',
            field=glitter.assets.fields.AssetForeignKey(on_delete=django.db.models.deletion.PROTECT, to='glitter_assets.Image'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.conf import settings

from mptt.forms import MPTTAdminForm

from glitter.integration import glitter_app_pool

from .models import Page


class DuplicatePageForm(forms.ModelForm):
    class Meta:
        model = Page
        fields = ['url', 'title', 'parent', 'login_required', 'show_in_navigation']
        labels = {
            'url': 'New URL',
            'title': 'New title',
        }

    def __init__(self, *args, **kwargs):
        if not getattr(settings, 'GLITTER_SHOW_LOGIN_REQUIRED', False):
            if 'login_required' in self.Meta.fields:
                self.Meta.fields.remove('login_required')
        super(DuplicatePageForm, self).__init__(*args, **kwargs)


def get_glitter_app_choices():
    glitter_apps = glitter_app_pool.get_glitter_apps()
    choices = [('', '(none)')]
    for app_system_name, glitter_app in glitter_apps.items():
        choices.append((app_system_name, glitter_app.name))

    # Sort by glitter_app name
    choices = sorted(choices, key=lambda x: x[1])
    return choices


class PageAdminForm(MPTTAdminForm):
    class Meta:
        model = Page
        widgets = {
            'glitter_app_name': forms.widgets.Select(choices=get_glitter_app_choices()),
        }
        fields = '__all__'



from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, modify_settings, override_settings

from glitter.blocks.html.models import HTML
from glitter.models import ContentBlock, Version
from glitter.page import Glitter
from glitter.pages.models import Page


class TestGlitterDefaultBlocks(TestCase):
    def setUp(self):
        self.page = Page.objects.create(url='/test/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page),
            object_id=self.page.id,
            template_name='glitter/sample.html',
        )
        self.glitter = Glitter(self.page_version)

    @override_settings(
        GLITTER_DEFAULT_BLOCKS=None,
    )
    @modify_settings(INSTALLED_APPS={
        'append': 'glitter.blocks.image',
    })
    def test_default_blocks(self):
        # Standard blocks: Text, Image, HTML
        self.assertEqual(
            self.glitter.default_blocks, [
                ('glitter_redactor.Redactor', 'Text'),
                ('glitter_image.ImageBlock', 'Image'),
                ('glitter_html.HTML', 'HTML'),
            ],
        )

    @override_settings(
        GLITTER_DEFAULT_BLOCKS=[('glitter_html.HTML', 'HTML')],
    )
    def test_custom_blocks(self):
        # Custom will return whatever is given to it
        self.assertEqual(self.glitter.default_blocks, [('glitter_html.HTML', 'HTML')])


class TestGlitterQueries(TestCase):
    def setUp(self):
        self.page = Page.objects.create(url='/test/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page),
            object_id=self.page.id,
            template_name='glitter/sample.html',
        )

    def test_html_blocks(self):
        # Add 100 HTML blocks to a page
        html_content_type = ContentType.objects.get_for_model(HTML)

        for block_position in range(1, 101):
            html_block = HTML.objects.create(content='<p>HTML Block</p>')
            content_block = ContentBlock.objects.create(
                obj_version=self.page_version,
                column='main_content',
                position=block_position,
                content_type=html_content_type,
                object_id=html_block.id,
            )
            html_block.content_block = content_block
            html_block.save(update_fields=['content_block'])

        # Two queries:
        # - One to select content blocks, along with their content types
        # - Another to select all HTML blocks
        with self.assertNumQueries(2):
            glitter = Glitter(page_version=self.page_version)
            glitter.render()



from celery import shared_task

from .utils import process_actions


@shared_task
def publish_task(pk=None):
    """
    Process a publish task from Celery.

    Can be given the ID of a task to action, or process all scheduled actions which need actioning.
    """
    action_ids = None

    if pk is not None:
        action_ids = [pk]

    process_actions(action_ids=action_ids)



INTERVAL_2_WEEKS = 1
INTERVAL_ONE_MONTH = 2
INTERVAL_THREE_MONTHS = 3
INTERVAL_SIX_MONTHS = 4
INTERVAL_ONE_YEAR = 5
INTERVAL_CHOICES = (
    (INTERVAL_2_WEEKS, 'Every 2 weeks'),
    (INTERVAL_ONE_MONTH, 'Every month'),
    (INTERVAL_THREE_MONTHS, 'Every 3 months'),
    (INTERVAL_SIX_MONTHS, 'Every 6 months'),
    (INTERVAL_ONE_YEAR, 'Every year'),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import get_callable
from django.test import TestCase
from django.test.client import RequestFactory
from django.test import modify_settings

from glitter.models import Version, ContentBlock
from glitter.pages.models import Page


from .models import RelatedPagesBlock


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
class RelatedPageTestCase(TestCase):
    def setUp(self):
        self.page = Page.objects.create(url='/related-page/', title='Test page')

        self.page_content_type = ContentType.objects.get_for_model(Page)

        self.editor = User.objects.create_user(username='relatedpage', password='relatedpage')

        page_version = Version.objects.create(
            content_type=self.page_content_type,
            object_id=self.page.id,
            template_name='glitter/sample.html',
            owner=self.editor
        )

        self.factory = RequestFactory()

        self.related_page_block = RelatedPagesBlock.objects.create()
        self.content_block = ContentBlock.objects.create(
            obj_version=page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(RelatedPagesBlock),
            object_id=self.related_page_block.id,
        )

        self.related_page_block.content_block = self.content_block
        self.related_page_block.save()

        self.request = self.factory.get('/')
        self.view = get_callable(RelatedPagesBlock.render_function)

    def test_view_with_block(self):
        self.view(
            self.related_page_block, self.request, False, self.content_block, 'test-class'
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import include, url
from django.contrib import admin

from glitter.blockadmin import blocks


urlpatterns = [
    # Django admin
    url(r'^admin/', include(admin.site.urls)),

    # Glitter block admin
    url(r'^blockadmin/', include(blocks.site.urls)),

]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0003_remove_empty_contentblocks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contentblock',
            name='object_id',
            field=models.PositiveIntegerField(),
        ),
    ]



import datetime

from django.contrib.admin.sites import AdminSite
from django.test import SimpleTestCase, modify_settings
from django.test.client import RequestFactory

from glitter.pages.admin import PageAdmin
from glitter.pages.models import Page
from glitter.reminders.admin import ReminderInline
from glitter.reminders.choices import INTERVAL_CHOICES
from glitter.reminders.models import Reminder


class MockSuperUser(object):
    id = 10

    def has_perm(self, perm):
        return True

    def has_module_perms(self, module):
        return True

    def is_active(self):
        return True

    def is_staff(self):
        return True


class ReminderModelTestCase(SimpleTestCase):

    def test_intervals_data(self):
        """
        Test interval make sure if new interval added we can easily get new timedelta for
        created interval.
        """
        for interval_id in dict(INTERVAL_CHOICES).keys():
            reminder = Reminder(interval=interval_id, content_type_id=1)
            self.assertIsInstance(
                reminder.get_interval_timedelta(), datetime.timedelta
            )


class ReminderAdminTestCase(SimpleTestCase):

    def setUp(self):
        self.request = RequestFactory().request()
        self.request.user = MockSuperUser()

    @modify_settings(INSTALLED_APPS={
        'remove': 'glitter.reminders',
    })
    def test_the_inline_not_appears(self):
        """ Test to make sure inline is not set if the settings variable is not set. """
        PageAdmin.inlines = []
        page_admin = PageAdmin(model=Page, admin_site=AdminSite())
        page_admin.get_inline_instances(self.request)
        self.assertNotIn(ReminderInline, page_admin.inlines)

    @modify_settings(INSTALLED_APPS={
        'append': 'glitter.reminders',
    })
    def test_the_inline_appears(self):
        """ Test to make sure inline is not set if the settings variable is not set. """
        PageAdmin.inlines = []
        page_admin = PageAdmin(model=Page, admin_site=AdminSite())
        page_admin.get_inline_instances(self.request)
        self.assertIn(ReminderInline, page_admin.inlines)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test import TestCase, Client
from django.test import override_settings

from glitter.blocks.html.models import HTML
from glitter.models import ContentBlock, Version
from glitter.pages.models import Page


@override_settings(
    TEMPLATE_DIRS=(os.path.join(os.path.dirname(__file__), 'templates'),),
    GLITTER_SHOW_LOGIN_REQUIRED=True,
)
class DuplicatePageTestCase(TestCase):
    def setUp(self):

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # User
        self.super_user = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.super_user_client = Client()
        self.super_user_client.login(username='test', password='test')

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.super_user
        )

        # Html block
        self.html_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html_block),
            object_id=self.html_block.id
        )
        self.html_block.content_block = self.content_block
        self.html_block.save()

        self.duplicate_page_view_url = reverse(
            'admin:glitter_pages_page_duplicate', args=(self.page.id,)
        )

    def test_duplication(self):
        data = {
            'url': '/newhome/',
            'title': 'Title'
        }
        self.super_user_client.post(self.duplicate_page_view_url, data)
        self.assertEqual(data['url'], Page.objects.get(url=data['url']).url)



import importlib
from threading import Lock
import sys

from django.http import Http404, HttpResponseRedirect

from glitter.exceptions import GlitterRedirectException, GlitterUnpublishedException
from glitter.pages.models import Page


class PageFallbackMiddleware(object):
    def process_response(self, request, response):

        # Import here as it causes migrations for pages if app is not installed apps.
        from glitter.pages.views import glitter, render_object_unpublished

        if response.status_code != 404:
            return response  # No need to check for a page for non-404 responses.

        try:
            return glitter(request=request, url=request.path_info)
        except Http404:
            # Return the original response if a 404 occurs
            return response
        except GlitterUnpublishedException as exception:
            return render_object_unpublished(request=request, obj=exception.obj)
        except GlitterRedirectException as exception:
            return HttpResponseRedirect(exception.url)

        return None


_urlconf_pages = []
_urlconf_lock = Lock()


class GlitterUrlConfMiddleware(object):
    def process_request(self, request):
        """
        Reloads glitter URL patterns if page URLs change.

        Avoids having to restart the server to recreate the glitter URLs being used by Django.
        """
        global _urlconf_pages

        page_list = list(
            Page.objects.exclude(glitter_app_name='').values_list('id', 'url').order_by('id')
        )

        with _urlconf_lock:
            if page_list != _urlconf_pages:
                glitter_urls = 'glitter.urls'
                if glitter_urls in sys.modules:
                    importlib.reload(sys.modules[glitter_urls])
                _urlconf_pages = page_list



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_related_pages', '0002_linkfield'),
    ]

    operations = [
        migrations.AddField(
            model_name='relatedpage',
            name='new_window',
            field=models.BooleanField(verbose_name='Open link in new window', default=False),
        ),
    ]



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.db.models.fields import BLANK_CHOICE_DASH

from glitter.models import Version

from .models import PublishAction


def object_version_choices(obj):
    """
    Return a list of form choices for versions of this object which can be published.
    """
    choices = BLANK_CHOICE_DASH + [(PublishAction.UNPUBLISH_CHOICE, 'Unpublish current version')]

    # When creating a new object in the Django admin - obj will be None
    if obj is not None:
        saved_versions = Version.objects.filter(
            content_type=ContentType.objects.get_for_model(obj),
            object_id=obj.pk,
        ).exclude(
            version_number=None,
        )

        for version in saved_versions:
            choices.append((version.version_number, version))

    return choices



from __future__ import unicode_literals

from datetime import timedelta

from django.core.validators import ValidationError
from django.test import SimpleTestCase
from django.utils import timezone

from glitter.publisher.validators import future_date


class TestFutureDateValidator(SimpleTestCase):
    def test_valid_date(self):
        next_week = timezone.now() + timedelta(weeks=1)

        date_valid = future_date(next_week)

        self.assertIsNone(date_valid)

    def test_invalid_past_date(self):
        last_week = timezone.now() - timedelta(weeks=1)

        with self.assertRaises(ValidationError):
            future_date(last_week)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from unittest import skipIf

from django.conf import settings
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test import TestCase, Client, modify_settings

from glitter.blockadmin.blocks import BlockAdminSite, BlockAdmin
from glitter.blocks.html.models import HTML
from glitter.models import ContentBlock, Version
from glitter.pages.admin import PageAdmin
from glitter.pages.models import Page


SAMPLE_BLOCK_MISSING = 'glitter.tests.sampleblocks' not in settings.INSTALLED_APPS
if not SAMPLE_BLOCK_MISSING:
    from glitter.tests.sampleblocks.admin import SampleInlineAdmin


class MockRequest(object):
    pass


class MockSuperUser(object):
    def has_perm(self, perm):
        return True

request = MockRequest()
request.user = MockSuperUser()


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sampleblocks',
    },
)
class TestBlockAdminSite(TestCase):
    def setUp(self):
        self.site = BlockAdminSite(name='block_admin')
        self.site.register(HTML)

        self.page_admin = PageAdmin(Page, AdminSite())

        # Superuser
        self.super_user = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.super_user_client = Client()
        self.super_user_client.login(username='test', password='test')

        # Normal user
        self.normal_user = User.objects.create_user('name', 'test1@test.com', 'psw')
        self.normal_user_client = Client()
        self.normal_user_client.login(username='name', password='psw')

    def test_register_with_admin_class(self):
        self.site.unregister(HTML)
        self.site.register(HTML, BlockAdmin)

    def test_register_block(self):
        self.site.register_block(HTML, 'Common')

        # Run again whe `Common` is added in self.block_list
        self.site.register_block(HTML, 'Common')

    def test_get_urls(self):
        with self.settings(DEBUG=True):
            self.site.get_urls()
        self.site.get_urls()

    def test_admin_view(self):
        # Cacheble
        jsi18n_url = reverse('block_admin:jsi18n')
        response = self.super_user_client.get(jsi18n_url)
        self.assertEqual(response.status_code, 200)

        # Not cacheble
        jsi18n_url = reverse('block_admin:jsi18n')
        response = self.super_user_client.get(jsi18n_url, {'cacheable': False})
        self.assertEqual(response.status_code, 200)

        response = self.normal_user_client.get(jsi18n_url)
        self.assertEqual(response.status_code, 403)


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sampleblocks',
    },
)
class TestBlockAdmin(TestCase):
    def setUp(self):

        # Superuser
        self.superuser = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.superuser_client = Client()
        self.superuser_client.login(username='test', password='test')

        # Normal user
        self.normal_user = User.objects.create_user('name', 'test1@test.com', 'psw')
        self.normal_user.is_staff = True
        self.normal_user.save()
        self.normal_user_client = Client()
        self.normal_user_client.login(username='name', password='psw')

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.superuser
        )
        self.html_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html_block),
            object_id=self.html_block.id
        )
        self.html_block.content_block = self.content_block
        self.html_block.save()

        self.site = AdminSite()
        self.model_admin = BlockAdmin(HTML, self.site)

        # Information about model
        self.info = self.model_admin.model._meta.app_label, self.model_admin.model._meta.model_name

        self.continue_view_url = reverse(
            'block_admin:%s_%s_continue' % self.info, args=(self.html_block.content_block.id,)
        )

        self.change_view_url = reverse(
            'block_admin:%s_%s_change' % self.info, args=(self.html_block.id,)
        )

    def test_change_form_template(self):
        self.model_admin.change_form_template

    def test_admin_view(self):
        jsi18n_url = reverse('block_admin:jsi18n')
        response = self.superuser_client.get(jsi18n_url)
        self.assertEqual(response.status_code, 200)

    def test_continue_view(self):
        response = self.superuser_client.get(self.continue_view_url)
        self.assertEqual(response.status_code, 200)

        continue_view_url_does_not_exit = reverse(
            'block_admin:%s_%s_continue' % self.info, args=('4')
        )
        response = self.superuser_client.get(continue_view_url_does_not_exit)
        self.assertEqual(response.status_code, 404)

    def test_continue_view_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.normal_user_client.get(self.continue_view_url)
        self.assertEqual(response.status_code, 403)

    def test_change_view(self):
        response = self.superuser_client.get(self.change_view_url)
        self.assertEqual(response.status_code, 200)

    def test_change_view_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.normal_user_client.get(self.change_view_url)
        self.assertEqual(response.status_code, 403)

        change_view_url_does_not_exit = reverse(
            'block_admin:%s_%s_change' % self.info, args=('4')
        )
        response = self.superuser_client.get(change_view_url_does_not_exit)
        self.assertEqual(response.status_code, 404)

    def test_change_view_page_version(self):
        self.page_version.version_number = 1
        self.page_version.save()
        response = self.superuser_client.get(self.change_view_url)
        self.assertEqual(response.status_code, 403)


@skipIf(SAMPLE_BLOCK_MISSING, 'glitter.tests.sampleblocks is not installed')
class TestInlineBlockAdmin(TestCase):
    def setUp(self):
        # Superuser
        self.superuser = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.superuser_client = Client()
        self.superuser_client.login(username='test', password='test')

        self.model_admin = SampleInlineAdmin(Page, AdminSite())

    def test_has_add_permission(self):
        request = MockRequest()
        request.user = self.superuser
        self.assertTrue(self.model_admin.has_add_permission(request))

    def test_has_change_permission(self):
        request = MockRequest()
        request.user = self.superuser
        self.assertTrue(self.model_admin.has_change_permission(request))

    def test_has_delete_permission(self):
        request = MockRequest()
        request.user = self.superuser
        self.assertTrue(self.model_admin.has_delete_permission(request))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.test import TestCase

from glitter.blockadmin.blocks import BlockAdminSite


class TestImageBlockAdmin(TestCase):
    fixtures = ['image.json']

    def setUp(self):
        self.site = BlockAdminSite(name='block_admin')

    def test_lazy_loading(self):
        url = reverse('block_admin:get-lazy-images')

        # Test if throws the errors without passing any arguments
        response = self.client.get(url)
        self.assertEqual(response.status_code, 400)

        # Test by passing the string for the `last_image_image`
        response = self.client.get(url, {'last_image_id': 'test'})
        self.assertEqual(response.status_code, 400)

        # Test to see if gets the image if the id passed is higher then the one available.
        response = self.client.get(url, {'last_image_id': 2})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(1, len(response.context['images']))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_banner', '0004_delete_empty_blocks'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bannerinline',
            options={'ordering': ('position', 'id'), 'verbose_name': 'banner'},
        ),
        migrations.AddField(
            model_name='bannerinline',
            name='position',
            field=models.PositiveIntegerField(db_index=True, default=0),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks


from .models import RelatedPage, RelatedPagesBlock


class RelatedPageInline(blocks.StackedInline):
    model = RelatedPage
    extra = 1


class RelatedPagesBlockAdmin(blocks.BlockAdmin):
    inlines = [RelatedPageInline]


blocks.site.register(RelatedPagesBlock, RelatedPagesBlockAdmin)
blocks.site.register_block(RelatedPagesBlock, 'App Blocks')



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType

import factory

from glitter.models import Version
from glitter.pages.models import Page


class PageFactory(factory.DjangoModelFactory):
    url = factory.Sequence(lambda n: '/page-{}/'.format(n))
    title = factory.Sequence(lambda n: 'Page {}'.format(n))

    class Meta:
        model = Page


class VersionFactory(factory.DjangoModelFactory):
    object_id = factory.SelfAttribute('content_object.id')
    content_type = factory.LazyAttribute(
        lambda o: ContentType.objects.get_for_model(o.content_object),
    )

    class Meta:
        exclude = ('content_object',)
        abstract = True


class PageVersionFactory(VersionFactory):
    content_object = factory.SubFactory(PageFactory)

    class Meta:
        model = Version

    @factory.post_generation
    def set_version(self, create, extracted, **kwargs):
        if extracted:
            page = self.content_object
            page.current_version = self

            if create:
                page.save()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from django.conf import settings
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import Permission, User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.test import TestCase, Client
from django.test import override_settings, modify_settings

from glitter.forms import MoveBlockForm
from glitter.blocks.html.models import HTML
from glitter.models import Version, ContentBlock
from glitter.pages.admin import PageAdmin
from glitter.pages.models import Page
from glitter.tests.sample.models import Book


SAMPLE_BLOCK_MISSING = 'glitter.tests.sampleblocks' not in settings.INSTALLED_APPS


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
@override_settings(
    TEMPLATE_DIRS=(os.path.join(os.path.dirname(__file__), 'templates'),),
    GLITTER_SHOW_LOGIN_REQUIRED=True,
)
class TestAdmin(TestCase):

    def setUp(self):

        # Permissions
        self.edit_permissions = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # Information about model
        self.info = self.page._meta.app_label, self.page._meta.model_name

        # Superuser
        self.super_user = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.super_user_client = Client()
        self.super_user_client.login(username='test', password='test')

        # Editor with editing permissions
        self.editor = User.objects.create_user('editor', 'editor@test.com', 'editor')
        self.editor.is_staff = True
        self.editor.user_permissions.add(self.edit_permissions)
        self.editor.save()
        self.editor_client = Client()
        self.editor_client.login(username='editor', password='editor')

        # Editor with not editing permissions
        self.editor_no_permissions = User.objects.create_user(
            'editor_no_perm', 'editor_no_perm@test.com', 'editor_no_perm'
        )
        self.editor_no_permissions.is_staff = True
        self.editor_no_permissions.save()
        self.editor_no_permissions_client = Client()
        self.editor_no_permissions_client.login(
            username='editor_no_perm', password='editor_no_perm'
        )

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor
        )

        self.page_admin = PageAdmin(Page, AdminSite())

        self.change_obj_url = reverse('admin:glitter_pages_page_change', args=(self.page.id,))
        self.add_obj_url = reverse('admin:glitter_pages_page_add')
        self.page_redirect_url = reverse('admin:glitter_pages_page_redirect', args=(self.page.id,))

    def test_data_for_change_and_add_response(self):
        response_data = {'url': '/testing2/', 'title': 'Testing2'}
        response_save_and_edit_data = {
            'url': '/testing223/', 'title': 'Testing2', '_saveandedit': True
        }
        response_save_and_continue_edit = {
            'url': '/testing223/', 'title': 'Testing2', '_continue': True
        }
        return response_data, response_save_and_edit_data, response_save_and_continue_edit

    def test_admin_url(self):
        self.page_admin.view_url(self.page)

    def test_unpublish_count(self):
        self.page_admin.admin_unpublished_count(self.page)


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
class TestPermissions(TestCase):
    def setUp(self):
        # Permissions for objects we're testing
        self.edit_page = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )
        self.publish_page = Permission.objects.get_by_natural_key(
            'publish_page', 'glitter_pages', 'page'
        )
        self.edit_book = Permission.objects.get_by_natural_key(
            'edit_book', 'sample', 'book'
        )
        self.publish_book = Permission.objects.get_by_natural_key(
            'publish_book', 'sample', 'book'
        )

        # Superuser
        self.superuser = User.objects.create_superuser(
            username='superuser', email='', password=None
        )

        # Editor with editing permissions
        self.editor = User.objects.create_user(username='editor', email='', password=None)
        self.editor.is_staff = True
        self.editor.save()
        self.editor.user_permissions.add(self.edit_page, self.edit_book)

        # Publisher with edit and publish permissions
        self.publisher = User.objects.create_user(username='publisher', email='', password=None)
        self.publisher.is_staff = True
        self.publisher.save()
        self.publisher.user_permissions.add(
            self.edit_page, self.publish_page, self.edit_book, self.publish_book
        )

        # Staff with no editing permissions
        self.staff = User.objects.create_user(username='staff', email='', password=None)
        self.staff.is_staff = True
        self.staff.save()

        # Page with an unsaved page version
        self.page = Page.objects.create(url='/test/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page),
            object_id=self.page.id,
            template_name='glitter/sample.html',
            owner=self.editor,
        )
        self.page_admin = PageAdmin(Page, AdminSite())

        # Sample model
        self.book = Book.objects.create(title='Hello')
        self.book_admin = PageAdmin(Book, AdminSite())

    def test_edit_permission(self):
        # Only people with glitter_pages.edit_page have edit permission
        request = HttpRequest()

        request.user = self.editor
        self.assertTrue(self.page_admin.has_edit_permission(request=request))

        request.user = self.staff
        self.assertFalse(self.page_admin.has_edit_permission(request=request))

    def test_edit_version(self):
        # Only the creator of an unsaved version can edit it
        request = HttpRequest()

        request.user = self.superuser
        self.assertFalse(self.page_admin.has_edit_permission(
            request=request, version=self.page_version
        ))

        request.user = self.editor
        self.assertTrue(self.page_admin.has_edit_permission(
            request=request, version=self.page_version
        ))

    def test_publish_permission(self):
        # Only people with glitter_pages.publish_page have publish permission
        request = HttpRequest()

        request.user = self.publisher
        self.assertTrue(self.page_admin.has_publish_permission(request=request))

        request.user = self.staff
        self.assertFalse(self.page_admin.has_publish_permission(request=request))

    def test_book_model(self):
        # Test that permissions work with different types of models
        request = HttpRequest()

        request.user = self.editor
        self.assertTrue(self.book_admin.has_edit_permission(request=request))

        request.user = self.publisher
        self.assertTrue(self.book_admin.has_publish_permission(request=request))

        request.user = self.staff
        self.assertFalse(self.book_admin.has_edit_permission(request=request))
        self.assertFalse(self.book_admin.has_publish_permission(request=request))


class BaseViewsCase(TestAdmin):

    def setUp(self):
        super(BaseViewsCase, self).setUp()

        self.html1_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html1_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html1_block),
            object_id=self.html1_block.id
        )
        self.html1_block.content_block = self.html1_content_block
        self.html1_block.save()

        self.html2_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html2_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=3,
            content_type=ContentType.objects.get_for_model(self.html2_block),
            object_id=self.html2_block.id
        )
        self.html2_block.content_block = self.html2_content_block
        self.html2_block.save()

        self.html3_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html3_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='side',
            position=4,
            content_type=ContentType.objects.get_for_model(self.html3_block),
            object_id=self.html3_block.id
        )
        self.html3_block.content_block = self.html3_content_block
        self.html3_block.save()

    def change_page_version(self):
        self.page_version.version_number = 1
        self.page_version.save()


class TestPageChangeTemplateView(BaseViewsCase):
    def setUp(self):
        super(TestPageChangeTemplateView, self).setUp()
        self.change_template_url = reverse(
            'admin:%s_%s_changetemplate' % self.info, args=(self.page_version.id,)
        )

    def test_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.editor_no_permissions_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 302)

    def test_post(self):
        # Check POST
        response = self.editor_client.post(self.change_template_url, {
            'template_name': 'glitter/sample2.html',
        })
        self.assertEqual(response.status_code, 302)

        # Check template gets changed
        page_version = Version.objects.get(id=self.page_version.id)
        self.assertEqual(page_version.template_name, 'glitter/sample2.html')

        # These blocks get moved to a new column
        html1_content_block = ContentBlock.objects.get(id=self.html1_content_block.id)
        html2_content_block = ContentBlock.objects.get(id=self.html2_content_block.id)

        self.assertEqual(html1_content_block.column, 'content')
        self.assertEqual(html2_content_block.column, 'content')
        self.assertEqual(html1_content_block.position, 1)
        self.assertEqual(html2_content_block.position, 2)

        # Index error if ContentBlock doesn't exist
        self.editor_client.post(self.change_template_url, {
            'template_name': 'glitter/sample.html',
        })

    def test_page_owner(self):
        """ Check page owner. """
        self.page_version.owner = self.super_user
        self.page_version.save()
        response = self.editor_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockAddView(BaseViewsCase):
    def setUp(self):
        super(TestPageBlockAddView, self).setUp()
        self.page_block_add_view_url = '{}?column=main_content'.format(reverse(
            'block_admin:%s_%s_add' % (HTML._meta.app_label, HTML._meta.model_name),
            kwargs={
                'version_id': self.page_version.id,
            }
        ))

    def test_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.editor_no_permissions_client.get(self.page_block_add_view_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.page_block_add_view_url)
        self.assertEqual(response.status_code, 200)

    def test_add_block(self):
        response = self.editor_client.post(self.page_block_add_view_url, {
            'content': '<p>Test</p>',
        })
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'admin/glitter/update_column.html')

    def test_add_block_top(self):
        response = self.editor_client.post(self.page_block_add_view_url + '&top=true', {
            'content': '<p>Test</p>',
        })
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'admin/glitter/update_column.html')

    def test_add_continue(self):
        response = self.editor_client.post(self.page_block_add_view_url, {
            'content': '<p>Test</p>',
            '_continue': '',
        }, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'blockadmin/continue.html')

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.page_block_add_view_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockDeleteView(BaseViewsCase):

    def setUp(self):
        super(TestPageBlockDeleteView, self).setUp()
        self.page_block_delete_view_url = reverse(
            'admin:%s_%s_block_delete' % self.info, args=(self.html1_content_block.id,)
        )

    def test_permissions(self):
        """ Test permissions. """
        response = self.editor_no_permissions_client.get(self.page_block_delete_view_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.page_block_delete_view_url)
        self.assertEqual(response.status_code, 200)

    def test_post(self):

        response = self.editor_client.post(self.page_block_delete_view_url, {
            'column': 'main_content',
            'block_type': u'glitter_html_block.HTML',
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.page_block_delete_view_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockMoveView(BaseViewsCase):
    def setUp(self):
        super(TestPageBlockMoveView, self).setUp()
        self.html1_block_move_view_url = reverse(
            'admin:%s_%s_block_move' % self.info, args=(self.html1_content_block.id,)
        )
        self.html2_block_move_view_url = reverse(
            'admin:%s_%s_block_move' % self.info, args=(self.html2_content_block.id,)
        )
        self.html3_block_move_view_url = reverse(
            'admin:%s_%s_block_move' % self.info, args=(self.html3_content_block.id,)
        )

    def test_permissions(self):
        """ Test permissions. """
        response = self.editor_no_permissions_client.get(self.html1_block_move_view_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.html1_block_move_view_url)
        self.assertEqual(response.status_code, 200)

    def test_html1_move_block_bottom(self):
        """ Move block top. """
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_BOTTOM,
        })
        self.assertEqual(response.status_code, 200)

    def test_html2_move_block_bottom(self):
        """ Move block up. """
        response = self.editor_client.post(self.html2_block_move_view_url, {
            'move': MoveBlockForm.MOVE_BOTTOM,
        })
        self.assertEqual(response.status_code, 200)

    def test_html3_move_block_bottom(self):
        """ Move block down. """
        response = self.editor_client.post(self.html3_block_move_view_url, {
            'move': MoveBlockForm.MOVE_BOTTOM
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html1_block_top(self):
        # Move block bottom
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_TOP
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html2_block_top(self):
        # Move block bottom
        response = self.editor_client.post(self.html2_block_move_view_url, {
            'move': MoveBlockForm.MOVE_TOP
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html1_block_up(self):
        # Move block bottom
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_UP
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html1_block_down(self):
        # Move block bottom
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_DOWN
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.html1_block_move_view_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockColumnView(BaseViewsCase):

    def setUp(self):
        super(TestPageBlockColumnView, self).setUp()
        self.html1_block_column_url = reverse(
            'admin:%s_%s_block_column' % self.info, args=(self.html1_content_block.id,)
        )

    def test_permissions(self):
        """ Test permissions. """
        response = self.editor_no_permissions_client.get(self.html1_block_column_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.html1_block_column_url)
        self.assertEqual(response.status_code, 200)

    def test_move_block_side(self):
        """ Move block side. """
        response = self.editor_client.post(self.html1_block_column_url, {
            'move': 'side',
        })
        self.assertEqual(response.status_code, 200)

    def test_get_last_block(self):
        self.html3_block.delete()
        self.html3_content_block.delete()

        response = self.editor_client.post(self.html1_block_column_url, {
            'move': 'side',
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.html1_block_column_url)
        self.assertEqual(response.status_code, 403)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .models import ContactFormBlock


blocks.site.register(ContactFormBlock)
blocks.site.register_block(ContactFormBlock, 'Forms')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from functools import update_wrapper

from django.apps import apps
from django.conf import settings
from django.conf.urls import url
from django.contrib import admin
from django.contrib.admin.options import csrf_protect_m
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.db import transaction
from django.http import HttpResponseRedirect
from django.shortcuts import render, get_object_or_404

from django_mptt_admin.admin import DjangoMpttAdmin
from mptt.admin import MPTTModelAdmin

from glitter.admin import GlitterAdminMixin
from glitter.models import Version

from .forms import DuplicatePageForm, PageAdminForm
from .models import Page


@admin.register(Page)
class PageAdmin(GlitterAdminMixin, DjangoMpttAdmin, MPTTModelAdmin):
    list_display = (
        'title', 'url', 'view_url', 'is_published', 'in_nav', 'admin_unpublished_count',
        'glitter_app_name',
    )
    mptt_level_indent = 25
    glitter_render = True
    change_list_template = 'admin/pages/page/change_list.html'
    change_form_template = 'admin/pages/page/change_form.html'
    form = PageAdminForm

    def get_fieldsets(self, request, obj=None):
        fields = [
            'url', 'title', 'parent', 'tags', 'login_required', 'show_in_navigation',
        ]

        # Don't show login_required unless needed
        if not getattr(settings, 'GLITTER_SHOW_LOGIN_REQUIRED', False):
            fields.remove('login_required')

        # Show glitter tags if it's set to show.
        if not getattr(settings, 'GLITTER_PAGES_TAGS', False):
            fields.remove('tags')

        fieldsets = [
            [None, {'fields': fields}],

            ['Advanced options', {
                'classes': ['collapse'],
                'fields': ['glitter_app_name'],
            }]
        ]
        return fieldsets

    @csrf_protect_m
    def changelist_view(self, request, extra_context=None):
        template_response = super(PageAdmin, self).changelist_view(request, extra_context)
        template_response.template_name = 'admin/pages/page/change_list_tree.html'
        return template_response

    def get_urls(self):
        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)
        urlpatterns = super(PageAdmin, self).get_urls()

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = [
            url(r'^$', wrap(self.grid_view)),
            url(r'^tree/$', wrap(self.changelist_view), name='%s_%s_tree' % info),
            url(r'^(\d+)/duplicate/$', wrap(self.duplicate_page), name='%s_%s_duplicate' % info),
        ] + urlpatterns
        return urlpatterns

    def get_inline_instances(self, request, obj=None):
        # Optional glitter applications. Both are imported after the is_installed check to prevent
        # migrations applied to the core glitter app
        if apps.is_installed('glitter.publisher'):
            from glitter.publisher.admin import ActionInline
            if ActionInline not in self.inlines:
                self.inlines = self.inlines + [ActionInline]

        if apps.is_installed('glitter.reminders'):
            from glitter.reminders.admin import ReminderInline
            if ReminderInline not in self.inlines:
                self.inlines = self.inlines + [ReminderInline]

        return super(PageAdmin, self).get_inline_instances(request, obj)

    def view_url(self, obj):
        info = self.model._meta.app_label, self.model._meta.model_name
        redirect_url = reverse('admin:%s_%s_redirect' % info, kwargs={'object_id': obj.id})
        return '<a href="%s">View page</a>' % (redirect_url)
    view_url.short_description = 'View page'
    view_url.allow_tags = True

    def in_nav(self, obj):
        return obj.show_in_navigation
    in_nav.boolean = True

    def admin_unpublished_count(self, obj):
        return obj.unpublished_count or ''
    admin_unpublished_count.short_description = 'Unpublished pages'
    admin_unpublished_count.allow_tags = True

    @csrf_protect_m
    @transaction.atomic
    def duplicate_page(self, request, obj_id):
        obj = get_object_or_404(Page, id=obj_id)

        if not self.has_add_permission(request):
            raise PermissionDenied

        if request.method == "POST":
            form = DuplicatePageForm(request.POST or None)
            if form.is_valid():
                new_page = form.save()

                # Use current version if exists if not get the latest
                if obj.current_version:
                    current_version = obj.current_version
                else:
                    current_version = obj.get_latest_version()

                if current_version:

                    # Create a new version
                    new_version = Version(content_object=new_page)
                    new_version.template_name = current_version.template_name
                    new_version.version_number = 1
                    new_version.owner = request.user
                    new_version.save()

                    self.duplicate_content(current_version, new_version)

                return HttpResponseRedirect(
                    reverse('admin:glitter_pages_page_change', args=(new_page.id,))
                )
        else:
            form = DuplicatePageForm(initial={
                'url': obj.url,
                'title': obj.title,
                'parent': obj.parent,
            })
        adminForm = admin.helpers.AdminForm(
            form=form,
            fieldsets=[('Duplicate Page: {}'.format(obj), {
                'fields': DuplicatePageForm.Meta.fields
            })],
            prepopulated_fields=self.get_prepopulated_fields(request, obj),
            readonly_fields=self.get_readonly_fields(request, obj),
            model_admin=self
        )
        context = {
            'adminform': adminForm,
            'opts': obj._meta,
            'change': False,
            'is_popup': False,
            'save_as': False,
            'has_delete_permission': self.has_delete_permission(request, obj),
            'has_add_permission': self.has_add_permission(request),
            'has_change_permission': self.has_change_permission(request, obj),
        }
        return render(
            request, 'admin/pages/page/duplicate_page.html', context
        )



from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, override_settings

from glitter.models import Version
from glitter.pages.models import Page
from glitter.reminders.admin import ReminderInline


class MockRequest(object):
    pass


class MockSuperUser(object):
    id = 10

    def has_perm(self, perm):
        return True

    def has_module_perms(self, module):
        return True

    def is_active(self):
        return True

    def is_staff(self):
        return True


@override_settings(
    ROOT_URLCONF='glitter.reminders.tests.urls',
)
class ReminderAdminTestCase(TestCase):

    def setUp(self):
        self.request = MockRequest()
        self.request.user = MockSuperUser()

    @classmethod
    def setUpTestData(cls):
        cls.superuser_password = 'secret'
        cls.superuser_username = 'super'
        cls.superuser_email = 'test@test.com'
        cls.superuser = User.objects.create_superuser(
            username=cls.superuser_username,
            password=cls.superuser_password,
            email=cls.superuser_email
        )

    def create_page_with_version(self):
        self.page = Page.objects.create(url='/test/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.superuser
        )
        self.page_version.generate_version()
        self.page.current_version = self.page_version
        self.page.save()

    def test_initial_data(self):
        """
        Check if user initial data been passed to the formset if page current object exists.
        """
        self.create_page_with_version()

        inline = ReminderInline(Page, AdminSite())
        formset = inline.get_formset(self.request, self.page)
        self.assertEqual(
            formset.form.base_fields['user'].initial, self.request.user.id
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_video', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='video',
            name='title',
            field=models.CharField(blank=True, help_text='Used for accessibility', max_length=150),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .models import HTML


blocks.site.register(HTML)
blocks.site.register_block(HTML, 'Common')



from __future__ import unicode_literals

from django.conf import settings
from django.contrib.admin.models import CHANGE, LogEntry
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import force_text, python_2_unicode_compatible

from glitter.models import Version

from .validators import future_date


@python_2_unicode_compatible
class PublishAction(models.Model):
    UNPUBLISH_CHOICE = -1

    # The glittered object we're linked to
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    # When/what we're updating, and by who
    scheduled_time = models.DateTimeField(validators=[future_date])
    publish_version = models.IntegerField()
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT)

    class Meta:
        verbose_name = 'Scheduled publishing'
        ordering = ('scheduled_time', 'id')

    def __str__(self):
        if self.publish_version == self.UNPUBLISH_CHOICE:
            return 'Unpublish at {}'.format(self.scheduled_time)
        else:
            return 'Publish version {} at {}'.format(self.publish_version, self.scheduled_time)

    def save(self, *args, **kwargs):
        super(PublishAction, self).save(*args, **kwargs)

        # Add a task
        if self.can_schedule_task():
            self.schedule_task()

    def can_schedule_task(self):
        """
        Can task sched
        """
        from .utils import celery_enabled  # avoid a circular import

        tasks_enabled = celery_enabled()
        return tasks_enabled

    def schedule_task(self):
        """
        Schedules this publish action as a Celery task.
        """
        from .tasks import publish_task

        publish_task.apply_async(kwargs={'pk': self.pk}, eta=self.scheduled_time)

    def get_version(self):
        """
        Get the version object for the related object.
        """
        return Version.objects.get(
            content_type=self.content_type,
            object_id=self.object_id,
            version_number=self.publish_version,
        )

    def _publish(self):
        """
        Process a publish action on the related object, returns a boolean if a change is made.

        Only objects where a version change is needed will be updated.
        """
        obj = self.content_object
        version = self.get_version()
        actioned = False

        # Only update if needed
        if obj.current_version != version:
            version = self.get_version()
            obj.current_version = version
            obj.save(update_fields=['current_version'])
            actioned = True

        return actioned

    def _unpublish(self):
        """
        Process an unpublish action on the related object, returns a boolean if a change is made.

        Only objects with a current active version will be updated.
        """
        obj = self.content_object
        actioned = False

        # Only update if needed
        if obj.current_version is not None:
            obj.current_version = None
            obj.save(update_fields=['current_version'])
            actioned = True

        return actioned

    def _log_action(self):
        """
        Adds a log entry for this action to the object history in the Django admin.
        """
        if self.publish_version == self.UNPUBLISH_CHOICE:
            message = 'Unpublished page (scheduled)'
        else:
            message = 'Published version {} (scheduled)'.format(self.publish_version)

        LogEntry.objects.log_action(
            user_id=self.user.pk,
            content_type_id=self.content_type.pk,
            object_id=self.object_id,
            object_repr=force_text(self.content_object),
            action_flag=CHANGE,
            change_message=message
        )

    def process_action(self):
        """
        Process the action and update the related object, returns a boolean if a change is made.
        """
        if self.publish_version == self.UNPUBLISH_CHOICE:
            actioned = self._unpublish()
        else:
            actioned = self._publish()

        # Only log if an action was actually taken
        if actioned:
            self._log_action()

        return actioned



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import get_callable, reverse
from django.test import TestCase
from django.test.client import RequestFactory
from django.test import modify_settings

from glitter.models import Version, ContentBlock
from glitter.pages.models import Page

from glitter.blocks.banner.models import Banner, BannerBlock, BannerInline
from glitter.blocks.banner.admin import BannerInlineAdmin


@modify_settings(INSTALLED_APPS={'append': 'glitter.tests.sample'})
class BannerTestCase(TestCase):
    def setUp(self):
        self.page = Page.objects.create(url='/redactor/', title='Test page')
        self.page_content_type = ContentType.objects.get_for_model(Page)

        editor = User.objects.create_superuser(
            username='editor', email='editor@example.org', password='editor',
        )
        self.client.login(username='editor', password='editor')

        page_version = Version.objects.create(
            content_type=self.page_content_type,
            object_id=self.page.id,
            template_name='glitter/sample.html',
            owner=editor,
        )

        self.factory = RequestFactory()

        self.banner = Banner.objects.create(
            title='Banner',
            link='www.blanc.ltd.uk'
        )
        self.banner_block = BannerBlock.objects.create()
        self.content_block = ContentBlock.objects.create(
            obj_version=page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(BannerBlock),
            object_id=self.banner_block.id,
        )
        self.banner_block.content_block = self.content_block
        self.banner_block.save()

        self.request = self.factory.get('/')
        self.view = get_callable(BannerBlock.render_function)

        self.inline = BannerInline.objects.create(
            banner_block=self.banner_block, banner=self.banner)

    def test_view_with_block(self):
        self.view(
            self.banner_block, self.request, False, self.content_block, 'test-class'
        )

    def test_model_string(self):
        self.assertEqual(str(self.banner), 'Banner')
        self.assertEqual(str(self.inline), 'Banner')

    def test_view_inline(self):
        site = AdminSite()
        view = BannerInlineAdmin(self.request, site)
        field = self.inline._meta.get_field('banner')
        form_field = view.formfield_for_dbfield(field)
        self.assertTrue(len(list(form_field.choices)) > 1)

    def test_banner_view_as_request(self):
        url = reverse('admin:glitter_banner_banner_change',
                      args=(self.banner.id,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_bannerblock_view_as_request(self):
        url = reverse('admin:glitter_banner_banner_change',
                      args=(self.banner.id,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_admin_bannerblock_change(self):
        url = reverse('block_admin:glitter_banner_bannerblock_change',
                      args=(self.banner_block.id,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



from django import forms

from .models import Reminder


class ReminderInlineAdminForm(forms.ModelForm):
    class Meta:
        model = Reminder
        fields = ('user', 'interval', 'object_id', 'content_type',)

    def clean_user(self):
        user = self.cleaned_data['user']
        if not user.email:
            raise forms.ValidationError(
                "User doesn't have an email address, please pick a different user or add an "
                "email address"
            )
        return user

    def validate_unique(self):
        """
        Add this method because django doesn't validate correctly because required fields are
        excluded.
        """
        unique_checks, date_checks = self.instance._get_unique_checks(exclude=[])
        errors = self.instance._perform_unique_checks(unique_checks)
        if errors:
            self.add_error(None, errors)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from glitter.models import Version, ContentBlock
from glitter.pages.models import Page

from .models import Redactor


class RedactorTestCase(TestCase):
    def setUp(self):
        page = Page.objects.create(url='/redactor/', title='Test page')

        self.page_content_type = ContentType.objects.get_for_model(Page)

        self.editor = User.objects.create_user(username='redactor', password='redactor')

        page_version = Version.objects.create(
            content_type=self.page_content_type, object_id=page.id,
            template_name='glitter/sample.html', owner=self.editor
        )
        self.redactor_block = Redactor.objects.create(
            content='Test'
        )

        self.content_block = ContentBlock.objects.create(
            obj_version=page_version,
            column='content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.redactor_block),
            object_id=self.redactor_block.id
        )
        self.redactor_block.content_block = self.content_block
        self.redactor_block.save()

    def test_existance(self):
        redactor = Redactor.objects.get(id=self.redactor_block.id)
        self.assertEqual(redactor.id, self.redactor_block.id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_pages', '0002_page_tags'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='glitter_app_name',
            field=models.CharField(verbose_name='Glitter App', db_index=True, max_length=255, blank=True),
        ),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.core.management.base import BaseCommand
from django.utils import timezone

from glitter.reminders.models import Reminder


class Command(BaseCommand):
    help = 'Management command to send reminder about out dated content.'

    def handle(self, *args, **options):

        self.verbosity = options.get('verbosity')

        for reminder in Reminder.objects.select_related(
                'content_type'
        ).exclude(
            user__email=''
        ).iterator():
            content_obj = reminder.content_object

            if content_obj.current_version and content_obj.published:

                date_difference = timezone.now() - content_obj.current_version.modified
                reminder_difference = timezone.now() - reminder.sent_at

                if (
                        date_difference >= reminder.get_interval_timedelta() and
                        reminder_difference >= reminder.get_interval_timedelta()
                ):

                    current_site = Site.objects.get_current()

                    send_mail(
                        subject='{site_name} - Outdated content for {model}'.format(
                            site_name=current_site.name,
                            model=content_obj._meta.model_name.title(),
                        ),
                        message=(
                            'Please update the outdated content for {model} '
                            'https://{domain}{url}'.format(
                                model=content_obj._meta.model_name.title(),
                                domain=current_site.domain,
                                url=content_obj.get_absolute_url(),
                            )
                        ),
                        from_email=settings.DEFAULT_FROM_EMAIL,
                        recipient_list=[reminder.user.email],
                    )
                    self.stdout.write(
                        'Email for {} is sent to: {}'.format(content_obj, reminder.user.email)
                    )
                    # Update the updated_at date to make sure it's not get send reminders.
                    reminder.sent_at = timezone.now()
                    reminder.save()
                else:
                    if self.verbosity == 3:
                        self.stdout.write(
                            'Email for {} is not sent to: {}'.format(
                                content_obj, reminder.user.email
                            )
                        )



from datetime import timedelta
try:
    from unittest import mock
except ImportError:
    import mock

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core import management
from django.test import TestCase, override_settings
from django.utils import six, timezone

from glitter.models import Version
from glitter.pages.models import Page
from glitter.reminders import choices
from glitter.reminders.models import Reminder


@override_settings(
    ROOT_URLCONF='glitter.reminders.tests.urls',
)
class ReminderManagementTestCase(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(
            username='joe', password='qwerty', email='test@gmail.com'
        )
        self.stdout = six.StringIO()
        self.stderr = six.StringIO()

    def tearDown(self):
        self.stdout.close()
        self.stderr.close()

    def create_page_with_version(self, modified):
        self.page = Page.objects.create(url='/test/', title='Test page')
        with mock.patch('django.db.models.fields.DateTimeField.pre_save') as mock_pre_save:
            mock_pre_save.return_value = modified
            self.page_version = Version.objects.create(
                content_type=ContentType.objects.get_for_model(Page),
                object_id=self.page.id,
                template_name='glitter/sample.html',
                owner=self.user
            )
            self.page_version.generate_version()
            self.page.current_version = self.page_version
            self.page.current_version.modified = mock.Mock(
                return_value=timezone.now() - timedelta(days=16)
            )
            self.page.save()

    def create_reminder(self, interval, page):
        self.reminder = Reminder.objects.create(
            interval=interval,
            content_type=ContentType.objects.get_for_model(Page),
            object_id=self.page.id,
            user=self.user
        )

    def test_send_reminder(self):
        """ Send reminder in two weeks interal. """
        # More then two weeks ago.
        modified_at = timezone.now() - timedelta(days=16)

        self.create_page_with_version(modified=modified_at)
        self.create_reminder(
            interval=choices.INTERVAL_2_WEEKS, page=self.page
        )
        self.reminder.sent_at = modified_at
        self.reminder.save()

        management.call_command('send_reminders', stdout=self.stdout, verbosity=3)
        command_output = self.stdout.getvalue().strip()

        self.assertEqual(
            command_output,
            'Email for {} is sent to: {}'.format(
                self.reminder.content_object, self.reminder.user.email
            )
        )

    def test_not_send_reminder_next_day(self):
        """
        Should not sent reminder if content updated 16 days ago but reminder was sent 1 day ago
        based on 2 weeks interval.
        """
        modified_at = timezone.now() - timedelta(days=16)

        self.create_page_with_version(modified=modified_at)
        self.create_reminder(
            choices.INTERVAL_2_WEEKS, self.page
        )
        self.reminder.sent_at = timezone.now() - timedelta(days=1)
        self.reminder.save()

        self.stdout = six.StringIO()

        management.call_command('send_reminders', stdout=self.stdout, verbosity=3)

        command_output = self.stdout.getvalue().strip()
        self.assertEqual(
            command_output,
            'Email for {} is not sent to: {}'.format(
                self.reminder.content_object, self.reminder.user.email
            )
        )

    def test_reminder_sent_at(self):
        """
        Test if sent_at set 100 days ago and the content was modified 3 days ago based on 2 weeks
        interval.
        """
        modified_at = timezone.now() - timedelta(days=3)

        self.create_page_with_version(modified=modified_at)
        self.create_reminder(choices.INTERVAL_2_WEEKS, self.page)

        self.reminder.sent_at = timezone.now() - timedelta(days=100)
        self.reminder.save()

        self.stdout = six.StringIO()
        management.call_command('send_reminders', stdout=self.stdout, verbosity=3)
        command_output = self.stdout.getvalue().strip()
        self.assertEqual(
            command_output,
            'Email for {} is not sent to: {}'.format(
                self.reminder.content_object, self.reminder.user.email
            )
        )



from glitter.pages.models import Page

from django.conf.urls import include, url
from django.db.utils import DatabaseError
from django.core.exceptions import FieldError
from glitter.integration import glitter_app_pool
import importlib

urlpatterns = []
used_apps = []

try:
    # Attempt to find all Glitter App Pages, get their corresponding Glitter App configs and then
    # use those to create URL patterns.
    app_pages = Page.objects.exclude(glitter_app_name='')
    for app_page in app_pages:
        glitter_app = glitter_app_pool.get_glitter_app(app_page.glitter_app_name)
        if glitter_app:
            app_url_conf = importlib.import_module(glitter_app.url_conf)

            # Django caches URLs quite hard, without the reload it continues to just use the
            # cached version.
            importlib.reload(app_url_conf)

            app_url = url(
                '^{}/'.format(app_page.url.strip('/')),
                include(app_url_conf, namespace=glitter_app.namespace)
            )
            urlpatterns.append(app_url)
            used_apps.append(app_page.glitter_app_name)

    # If a page has not been created for an app yet, we don't want a NoReverseMatch error every
    # time someone tries to '{% url %}' or 'reverse()' a viewname. So lets add a URL pattern entry
    # to support those requests.
    glitter_apps = glitter_app_pool.get_glitter_apps()
    for system_name, glitter_app in glitter_apps.items():
        if system_name not in used_apps:
            app_url_conf = importlib.import_module(glitter_app.url_conf)

            app_url = url(
                '^{}/'.format(system_name),
                include(app_url_conf, namespace=glitter_app.namespace)
            )
            urlpatterns.append(app_url)

except DatabaseError:
    # Database not setup correctly, not much we can do
    pass

except FieldError:
    # Likely that migrations to support Glitter Apps have not been executed. Not much we can do.
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Reminder',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('interval', models.IntegerField(choices=[(1, 'Every 2 weeks'), (2, 'Every month'), (3, 'Every 3 months'), (4, 'Every 6 months'), (5, 'Every year')])),
                ('sent_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='reminder',
            unique_together=set([('user', 'object_id', 'content_type')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from glitter.blockadmin import blocks

from .models import Video


blocks.site.register(Video)
blocks.site.register_block(Video, 'Media')



import datetime
from calendar import monthrange

from dateutil.relativedelta import relativedelta

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils import timezone

from . import choices as reminders_choices


class Reminder(models.Model):
    object_id = models.PositiveIntegerField()
    content_type = models.ForeignKey(ContentType)
    content_object = GenericForeignKey('content_type', 'object_id')
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    interval = models.IntegerField(choices=reminders_choices.INTERVAL_CHOICES)
    sent_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'object_id', 'content_type',)

    def __str__(self):
        return '{model} {obj} interval - {interval}'.format(
            model=self.content_type.model.title(),
            obj=self.content_object,
            interval=self.get_interval_display(),
        )

    def get_interval_timedelta(self):
        """ Spits out the timedelta in days. """

        now_datetime = timezone.now()
        current_month_days = monthrange(now_datetime.year, now_datetime.month)[1]

        # Two weeks
        if self.interval == reminders_choices.INTERVAL_2_WEEKS:
            interval_timedelta = datetime.timedelta(days=14)

        # One month
        elif self.interval == reminders_choices.INTERVAL_ONE_MONTH:
            interval_timedelta = datetime.timedelta(days=current_month_days)

        # Three months
        elif self.interval == reminders_choices.INTERVAL_THREE_MONTHS:
            three_months = now_datetime + relativedelta(months=+3)
            interval_timedelta = three_months - now_datetime

        # Six months
        elif self.interval == reminders_choices.INTERVAL_SIX_MONTHS:
            six_months = now_datetime + relativedelta(months=+6)
            interval_timedelta = six_months - now_datetime

        # One year
        elif self.interval == reminders_choices.INTERVAL_ONE_YEAR:
            one_year = now_datetime + relativedelta(years=+1)
            interval_timedelta = one_year - now_datetime

        return interval_timedelta



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.test import SimpleTestCase, TestCase, override_settings
from django.utils import timezone

from glitter.models import Version
from glitter.pages.models import Page
from glitter.publisher.models import PublishAction
from glitter.publisher.utils import process_actions, celery_enabled


@override_settings(GLITTER_PUBLISHER_CELERY=False)
class TestProcessActions(TestCase):
    @classmethod
    def setUpTestData(cls):
        user = User.objects.create(
            username='admin',
        )
        page = Page.objects.create(
            url='/test/',
            title='Test',
        )
        page_content_type = ContentType.objects.get_for_model(page)

        Version.objects.create(
            content_type=page_content_type,
            object_id=page.pk,
            template_name='demo.html',
            version_number=1,
        )

        cls.action_1 = PublishAction.objects.create(
            content_type=page_content_type,
            object_id=page.pk,
            scheduled_time=timezone.now(),
            publish_version=1,
            user=user,
        )

        cls.action_2 = PublishAction.objects.create(
            content_type=page_content_type,
            object_id=page.pk,
            scheduled_time=timezone.now(),
            publish_version=1,
            user=user,
        )

    def test_all_actions(self):
        actions = process_actions()

        self.assertEqual(actions, 2)
        self.assertEqual(PublishAction.objects.count(), 0)

    def test_one_action(self):
        actions = process_actions(action_ids=[self.action_1.id])

        self.assertEqual(actions, 1)
        self.assertEqual(PublishAction.objects.count(), 1)


class TestCeleryEnabled(SimpleTestCase):
    @override_settings(GLITTER_PUBLISHER_CELERY=True)
    def test_celery_enabled(self):
        enabled = celery_enabled()

        self.assertTrue(enabled)

    @override_settings(GLITTER_PUBLISHER_CELERY=False)
    def test_celery_disabled(self):
        enabled = celery_enabled()

        self.assertFalse(enabled)



from django.contrib.contenttypes.admin import GenericTabularInline
from django.utils.html import mark_safe

from .forms import ReminderInlineAdminForm
from .models import Reminder


class ReminderInline(GenericTabularInline):
    raw_id_fields = ('user',)
    model = Reminder
    form = ReminderInlineAdminForm
    verbose_name_plural = mark_safe('Reminder <small>(to update the content)</small>')
    extra = 0

    def get_formset(self, request, obj=None, **kwargs):
        """ Default user to the current version owner. """
        data = super(ReminderInline, self).get_formset(request, obj, **kwargs)
        if obj:
            data.form.base_fields['user'].initial = request.user.id
        return data



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from glitter.models import Version
from glitter.pages.models import Page
from glitter.publisher.models import PublishAction


class TestPublishAction(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = User.objects.create(
            username='admin',
        )
        cls.page = Page.objects.create(
            url='/test/',
            title='Test',
        )
        cls.page_content_type = ContentType.objects.get_for_model(cls.page)

        cls.version_1 = Version.objects.create(
            content_type=cls.page_content_type,
            object_id=cls.page.pk,
            template_name='demo.html',
            version_number=1,
        )
        cls.version_2 = Version.objects.create(
            content_type=cls.page_content_type,
            object_id=cls.page.pk,
            template_name='demo.html',
            version_number=2,
        )

    def setUp(self):
        self.page.refresh_from_db()

    def test_publish(self):
        obj = PublishAction(
            content_type=self.page_content_type,
            object_id=self.page.pk,
            publish_version=1,
            user=self.user,
        )

        actioned = obj.process_action()

        self.page.refresh_from_db()
        self.assertTrue(actioned)
        self.assertEqual(self.page.current_version, self.version_1)

    def test_publish_no_action(self):
        self.page.current_version = self.version_1
        self.page.save()
        obj = PublishAction(
            content_type=self.page_content_type,
            object_id=self.page.pk,
            publish_version=1,
            user=self.user,
        )

        actioned = obj.process_action()

        self.page.refresh_from_db()
        self.assertEqual(self.page.current_version, self.version_1)
        self.assertFalse(actioned)

    def test_unpublish(self):
        self.page.current_version = self.version_1
        self.page.save()
        obj = PublishAction(
            content_type=self.page_content_type,
            object_id=self.page.pk,
            publish_version=PublishAction.UNPUBLISH_CHOICE,
            user=self.user,
        )

        actioned = obj.process_action()

        self.page.refresh_from_db()
        self.assertTrue(actioned)
        self.assertIsNone(self.page.current_version)

    def test_unpublish_no_action(self):
        obj = PublishAction(
            content_type=self.page_content_type,
            object_id=self.page.pk,
            publish_version=PublishAction.UNPUBLISH_CHOICE,
            user=self.user,
        )

        actioned = obj.process_action()

        self.page.refresh_from_db()
        self.assertFalse(actioned)
        self.assertIsNone(self.page.current_version)
